/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUSDATASERVER_H
#define CONEXUSDATASERVER_H

#include <set>
#include <queue>

#include <sys/select.h>
#include <pthread.h>

#include <glibmm/dispatcher.h>
#include <glibmm/thread.h>

#include <conexus/object.h>
#include <conexus/data.h>

namespace Conexus {

  typedef enum ReadWrite {
    READ = 0x01,
    WRITE = 0x02,
    READWRITE = 0x03
  } ReadWrite;

  /**
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class DataServer : public Object {
    public:
      DataServer();

      virtual ~DataServer();

      virtual void start( ReadWrite rw = READ, bool use_dispatcher = false );
      virtual void stop( ReadWrite rw = READ );

      sigc::signal<void, Data> signal_data();
      sigc::signal<void, CData> signal_cdata();

      sigc::signal<void, size_t> signal_data_received();
      sigc::signal<void, size_t> signal_data_transmitted();

      virtual bool is_running();
      virtual bool is_read_running();
      virtual bool is_write_running();

      virtual const std::string& object_type() { static std::string s("Conexus::DataServer"); return s; }

    protected:
      bool m_running[ 2 ];
      pthread_t m_thread[ 2 ];
      pthread_attr_t m_thread_attr[ 2 ];

      pthread_mutex_t m_mutex_queue[ 2 ];
      pthread_mutex_t m_mutex_can_stop[ 2 ];
      pthread_mutex_t m_mutex_can_start[ 2 ];

      Glib::Dispatcher* m_dispatcher;
      bool m_use_dispatcher;
      sigc::connection m_dispatcher_connection;

      sigc::signal<void, Data> m_signal_data;
      sigc::signal<void, CData> m_signal_cdata;
      sigc::signal<void, size_t> m_signal_data_received;
      sigc::signal<void, size_t> m_signal_data_transmitted;
      std::queue<Data> m_read_queue;
      std::queue<Data> m_write_queue;

      virtual void read_thread_main();
      virtual void read_thread_cleanup();
      virtual void write_thread_main();
      virtual void write_thread_cleanup();

      void queue_received_data( Data d );
      void queue_write_data( Data d );
      void emit_received_data( );

    private:
      static void* read_thread_main_static( void* );
      static void* write_thread_main_static( void* );

  };

}

#endif
