/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "endpointserver.h"

#include <iostream>

namespace Conexus {

EndpointServer::EndpointServer()
 : Server()
{
  pthread_mutex_init(&m_mutex_signal_endpoint, NULL);
  pthread_mutex_init(&m_mutex_endpoint_queue, NULL);
}


EndpointServer::~EndpointServer()
{
  if (m_running)
    stop();
}


void EndpointServer::start(bool use_dispatcher)
{
  Server::start(use_dispatcher);

  if (m_dispatcher && ! m_dispatcher_connection)
    m_dispatcher_connection = m_dispatcher->connect(sigc::mem_fun(*this, &EndpointServer::emit_endpoints));
}

void EndpointServer::stop()
{
  if (m_dispatcher_connection)
    m_dispatcher_connection.disconnect();

  Server::stop();
}

sigc::signal<void, Endpoint::pointer> EndpointServer::signal_new_endpoint() {
  return m_signal_new_endpoint;
}

void EndpointServer::queue_endpoint( Endpoint::pointer endpoint )
{
  pthread_mutex_lock(&m_mutex_endpoint_queue);
  m_endpoint_queue.push(endpoint);
  pthread_mutex_unlock(&m_mutex_endpoint_queue);
  if ( pthread_mutex_trylock(&m_mutex_signal_endpoint) == 0 )
  {
    pthread_mutex_unlock(&m_mutex_signal_endpoint);
    if (m_use_dispatcher)
      m_dispatcher->emit();
    else
      emit_endpoints();
  }
}

void EndpointServer::emit_endpoints( )
{
  Endpoint::pointer current;
  pthread_mutex_lock(&m_mutex_signal_endpoint);
  pthread_mutex_lock(&m_mutex_endpoint_queue);
  while ( ! m_endpoint_queue.empty() )
  {
    current = m_endpoint_queue.front();
    m_endpoint_queue.pop();
    pthread_mutex_unlock(&m_mutex_endpoint_queue);
    m_signal_new_endpoint.emit(current);
    pthread_mutex_lock(&m_mutex_endpoint_queue);
  }
  pthread_mutex_unlock(&m_mutex_endpoint_queue);
  pthread_mutex_unlock(&m_mutex_signal_endpoint);
}

}
