/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "file.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace Conexus
  {

  File::File(const std::string& filename, unsigned long readwrite):
      RWFileDescriptor(readwrite)
  {
    this->set_filename(filename);
  }

  File::pointer File::create(const std::string& filename, unsigned long readwrite) {
    return pointer( new File(filename, readwrite) );
  }

  File::~File()
  {}

  void File::open( const std::string name, int state ) throw (open_error)
  {
    int flags=0;

    if ( is_open() )
      change_state(CLOSED);

    if ( state & (READ|WRITE) )
      flags |= O_RDWR;
    else if ( state & READ )
      flags |= O_RDONLY;
    else if ( state & WRITE )
      flags |= O_WRONLY;

    if ( state & APPEND )
      flags |= O_APPEND;
    if ( state & CREATE )
      flags |= O_CREAT;
    if ( state & DIRECT )
      flags |= O_DIRECT;
    if ( state & LARGE_FILE )
      flags |= O_LARGEFILE;
    if ( state & NO_ACCESS_TIME )
      flags |= O_NOATIME;
    if ( state & NO_FOLLOW )
      flags |= O_NOFOLLOW;
    if ( state & TRUNCATE )
      flags |= O_TRUNC;
    if ( state & EXCLUSIVE_CREATE )
      flags |= O_EXCL;

    m_fd = ::open(name.c_str(), flags);
    // TODO come back and check error codes
    set_state_opened();
  }

  void File::open( ) throw( open_error )
  {
    if (m_filename == std::string())
      throw(error::open::no_filename());
    open(m_filename, state() );
  }

  std::string File::filename( )
  {
    return m_filename;
  }

  void File::set_filename( std::string d, long unsigned state ) throw (open_error)
  {
    // if the filename is the same and the state is the same, then leave
    if (m_filename == d && (m_state==state || state==(unsigned)UNCHANGED))
      return;

    m_filename = d;
    m_signal_filename_changed.emit();
    change_state(state);
  }

  sigc::signal<void> File::signal_filename_changed()
  {
    return m_signal_filename_changed;
  }

}
