/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSFILE_H
#define CONEXUSFILE_H

#include <conexus/rwfiledescriptor.h>

/**
 * \page manual_file File Endpoints
 */

namespace Conexus
  {

  /**
   * This class encapsulates a file.
   * 
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class File : public RWFileDescriptor
    {
    public:
      typedef enum FileState {
        APPEND=LASTRWFILEDESCRIPTORSTATE<<1,
        CREATE=LASTRWFILEDESCRIPTORSTATE<<2,
        DIRECT=LASTRWFILEDESCRIPTORSTATE<<3,
        LARGE_FILE=LASTRWFILEDESCRIPTORSTATE<<4,
        NO_ACCESS_TIME=LASTRWFILEDESCRIPTORSTATE<<5,
        NO_FOLLOW=LASTRWFILEDESCRIPTORSTATE<<6,
        TRUNCATE=LASTRWFILEDESCRIPTORSTATE<<7,
        EXCLUSIVE_CREATE=LASTRWFILEDESCRIPTORSTATE<<8,
        LASTRWFILEDESCRIPTORSTATE=EXCLUSIVE_CREATE,
      } FileState;

      typedef boost::shared_ptr<File> pointer;
      
      File(const std::string& filename=std::string(), unsigned long readwrite=READ|WRITE|CREATE);
      
      static File::pointer create(const std::string& filename, unsigned long readwrite=READ|WRITE|CREATE);

      virtual ~File();

      /**
       * Opens the named file in the specified mode.
       */
      virtual void open(const std::string name, int state=READ|WRITE|CREATE) throw (open_error);

      /**
       * Will attempt to open the file if a filename has been specified. Otherwise an
       * error will be thrown.
       */
      virtual void open() throw (open_error);

      /** Will return the filename. */
      std::string filename();

      /**
       * Sets the filename.
       *
       * If the file is currently open and d is not equal to the current filename,
       * the current file will be closed.
       *
       * Also, if auto_open is set, the new file will be immediately opened.
       *
       * Additionally, this method checks whether the parameter d is the same as the current filename,
       * and parameter mode is the same as the current mode.
       * If both are equal, the file will not be closed and reopened unless force=true.
       */
      void set_filename(std::string d, long unsigned state=READ|WRITE|CREATE) throw (open_error);

      sigc::signal<void> signal_filename_changed();
      
      virtual const std::string& object_type() { static std::string s("Conexus::File"); return s; }

      protected:
      std::string m_filename;

      sigc::signal<void> m_signal_filename_changed;
    };

}

#endif
