/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "ipv6_tcpserver.h"

#include "ipv6_tcp.h"

namespace Conexus
  {

  namespace IPv6
    {

    TCPServer::TCPServer(uint16_t port):
        m_tcp_listener(PF_INET6, SOCK_STREAM, IPPROTO_TCP)
    {
      m_timeval.tv_sec = 0;
      m_timeval.tv_usec = 5000;
      m_local_interface.set_port(port);
    }

    TCPServer::pointer TCPServer::create( uint16_t port ) {
      return pointer( new TCPServer( port ) );
    }

    TCPServer::~TCPServer()
    {}

    Address& TCPServer::local_interface() {
      return m_local_interface;
    }

    void TCPServer::set_local_interface(Address addr) {
      m_local_interface = addr;
    }

    void TCPServer::start(bool use_dispatcher)
    {
      m_tcp_listener.bind(m_local_interface);
      m_tcp_listener.listen(5);
      Conexus::EndpointServer::start(use_dispatcher);
    }

    void TCPServer::stop()
    {
      Conexus::EndpointServer::stop();
      m_tcp_listener.close();
    }

    void TCPServer::set_responsiveness(long r)
    {
      m_timeval.tv_usec = r;
    }

    long TCPServer::responsiveness()
    {
      return m_timeval.tv_usec;
    }

    void TCPServer::service_thread_main()
    {
      int n;
      Data d, dcpy;
      std::set<FileDescriptor*>::iterator i;
      struct timeval tv;
      fd_set readfd;
      int fd;
      int new_connection;
      struct sockaddr addr;
      socklen_t addrlen;
      TCP::pointer newtcp;

      fd = m_tcp_listener.fd();
      
      if (fd < 0)
        throw;

      while (!m_terminate)
        {
          FD_ZERO(&readfd);
          FD_SET(fd, &readfd);
      
          if (m_timeval.tv_usec <= 0)
            n = select(fd+1, &readfd, NULL, NULL, NULL);
          else
            {
              tv = m_timeval;
              n = select(fd+1, &readfd, NULL, NULL, &tv);
            }
            if (FD_ISSET(fd, &readfd)) {
              new_connection = accept(fd, &addr, &addrlen);
              newtcp = TCP::create( new_connection, true );
              queue_endpoint( newtcp );
            }
        }
      m_running = false;
    }


  } // namespace IPv6

} // namespace Conexus
