/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef IPV6IPV6_TCPSERVER_H
#define IPV6IPV6_TCPSERVER_H

#include <conexus/ipv6_address.h>
#include <conexus/endpointserver.h>
#include <conexus/socket.h>

namespace Conexus
  {

  namespace IPv6
    {

    /**
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     */
    class TCPServer: public Conexus::EndpointServer
      {
      public:
        typedef boost::shared_ptr<TCPServer> pointer;
        
        TCPServer(uint16_t port=0);

        static TCPServer::pointer create(uint16_t port=0);

        virtual ~TCPServer();

        virtual Address& local_interface();

        virtual void set_local_interface(Address addr);

        virtual void start(bool use_dispatcher=false);
        virtual void stop();

        /**
         * Sets the responsiveness to add_*, remove_* and stop commands in microseconds;
         * lower is more responsive; larger is less responsive; default is 5000 microseconds (or 5 ms).
         *
         * In general this refers to the timeout period of the @em select() call used internally.
         * Each cycle of the server I/O thread acquires several mutex locks at the beginning
         * of the cycle and releases them at the end of the cycle.
         *
         * This value shortens the cycle period, thereby making the server more responsive to commands at
         * the cost of increasing overhead.
         */
        void set_responsiveness(long r);

        /**
         * Returns the server responsiveness level.
         */
        long responsiveness();

        virtual const std::string& object_type() { static std::string s("Conexus::IPv6::TCPServer"); return s; }

      protected:
        struct timeval m_timeval;
        Conexus::Socket m_tcp_listener;
        Address m_local_interface;


        virtual void service_thread_main();


      };

  } // namespace IPv6

} // namespace Conexus

#endif
