/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "pipe.h"

#include <unistd.h>

namespace Conexus
  {

  Pipe::Pipe():
      FileDescriptor(),
      m_write_fd(-1)
  {}

      Pipe::pointer Pipe::create() {
        return pointer( new Pipe() );
      }
      
  Pipe::~Pipe()
  {}

  void Pipe::open( ) throw( open_error )
  {
    int fds[2];
    int result;

    result = pipe(fds);

    switch (result)
      {
      case 0:
        break;
      case EFAULT:
        break;
      case EMFILE:
        break;
      case ENFILE:
        break;
      }

    m_fd = fds[0];
    m_write_fd = fds[1];
    set_state_opened();
    m_state |= READ|WRITE;
  }

  void Pipe::close(bool force) throw (close_error)
  {
    int i;
    if ( m_state & READ )
      {
        i = ::close(m_fd);
        if (i == -1)
          throw_close_error(errno);
      }

    if ( m_state & WRITE )
      {
        i = ::close(m_write_fd);
        if (i == -1)
          throw_close_error(errno);
      }

    set_state_closed();
  }

  void Pipe::close_read(bool force) throw (close_error)
  {
    int i;
    if ( m_state & READ )
      {
        i = ::close(m_fd);
        if (i == -1)
          throw_close_error(errno);
        m_state &= ~READ;
      }
  }

  void Pipe::close_write(bool force) throw (close_error)
  {
    int i;
    if ( m_state & WRITE )
      {
        i = ::close(m_write_fd);
        if (i == -1)
          throw_close_error(errno);
        m_state &= ~WRITE;
      }
  }

  ssize_t Pipe::write( const void * data, size_t size, IOMETHOD block ) throw (write_error)
  {
    if (is_write_blocked())
      return 0;

    if ( is_closed() )
      throw error::write::not_opened();

    return ::write(m_write_fd, data, size);
    // TODO come back and check error codes
  }

  Data Pipe::read( size_t s ) throw (read_error)
  {
    Data d;
    if (is_read_blocked())
      return d;

    if ( is_closed() )
      throw error::read::not_opened();

    if (s == 0)
      s = PIPE_BUF;

    d.data = Data::Octets(new Data::Octet[s]);
    d.size = ::read(m_fd, d.data.get(), s);

    if (d.size == 0)
      throw ( error::read::disconnected() );

    // TODO come back and check error codes
    return d;
  }

}
