/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSRWFILEDESCRIPTOR_H
#define CONEXUSRWFILEDESCRIPTOR_H

#include <conexus/filedescriptor.h>

namespace Conexus
  {

  /**
   * This class adds read/write management to a plain file descriptor object.
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class RWFileDescriptor : public FileDescriptor
    {
    public:
      /**
       * The serial port open modes. These have been selected so that masks may be created through
       * the bitwise operators.
       */
      typedef enum RWFileDescriptorState {
        READ=LASTENDPOINTSTATE<<1,           /**< opened for reading */
        WRITE=LASTENDPOINTSTATE<<2,            /**< opened for writing */
        LASTRWFILEDESCRIPTORSTATE=WRITE,
      } RWFileDescriptorState;

      RWFileDescriptor(unsigned long readwrite=READ|WRITE);

      virtual ~RWFileDescriptor();

      /**
       * Overrides parent's pure virtual method to provide for writing to the serial port.
       */
      virtual ssize_t write(const void* data, size_t size, IOMETHOD block=BLOCK) throw (write_error);

      /**
       * Overrides parent's pure virtual method to provide for reading from the serial port.
       */
      virtual Data read(size_t s = 0) throw (read_error);

      virtual void change_state(long unsigned new_state) throw (state_error);

      sigc::signal<void,bool> signal_readable_changed();

      sigc::signal<void,bool> signal_writable_changed();

      bool is_readable();
      bool is_writable();

      virtual const std::string& object_type() { static std::string s("Conexus::RWFileDescriptor"); return s; }

    protected:
      sigc::signal<void,bool> m_signal_readable_changed;
      sigc::signal<void,bool> m_signal_writable_changed;

      void set_state_readable(bool b);
      void set_state_writable(bool b);
    };

}

#endif
