/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "serial.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

namespace Conexus
  {

  Serial::Serial(const std::string& device, unsigned long readwrite):
      RWFileDescriptor(readwrite)
  {
    this->set_device(device);
  }

  Serial::pointer Serial::create(const std::string& device, unsigned long readwrite) {
    return pointer( new Serial( device, readwrite ) );
  }

  Serial::~Serial()
  {}

  void Serial::open( const std::string name, int state ) throw (open_error)
  {
    int flags=O_NOCTTY|O_NDELAY;

    if ( is_open() )
      change_state(CLOSED);

    if ( state & (READ|WRITE) )
      flags |= O_RDWR;
    else if ( state & READ )
      flags |= O_RDONLY;
    else if ( state & WRITE )
      flags |= O_WRONLY;

    m_fd = ::open(name.c_str(), flags);
    // TODO come back and check error codes
    set_state_opened();
  }

  void Serial::open( ) throw( open_error )
  {
    if (m_device == std::string())
      throw(error::open::no_device());
    open(m_device, state() );
  }

  std::string Serial::device( )
  {
    return m_device;
  }

  void Serial::set_device( const std::string& d, long unsigned state ) throw (open_error)
  {
    long unsigned tempstate;
    
    // if the device is the same and the state is the same, then leave
    if (m_device == d && (m_state==state || state==UNCHANGED))
      return;

    // set the new device
    m_device = d;
    
    // check to see what the new state should be - unchanged or the parameter
    if ( state == UNCHANGED )
      tempstate = m_state;
    else
      tempstate = state;
    
    // close the old file (changes the state so this needs to be after check above)
    this->close();
    
    // only potentially reopen the file if the device is not an empty string
    if ( m_device.size() > 0 )
      this->change_state(tempstate);
    
    m_signal_device_changed.emit();
  }

  sigc::signal<void> Serial::signal_device_changed()
  {
    return m_signal_device_changed;
  }

  void Serial::start(ReadWrite rw, bool use_dispatcher) {
    if ( is_closed() && m_device != std::string() )
      this->open();
    DataServer::start(rw, use_dispatcher);
  }
  
}
